/*
 * Copyright 2017-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.onosproject.fpcagent.protocols;

import org.onlab.packet.Ip4Address;

import java.math.BigInteger;

/**
 * Communication Service which sends packets to desired DPNs.
 */
public interface DpnCommunicationService {

    /**
     * Creates Mobility Session.
     *
     * @param topicId     - DPN Topic ID
     * @param imsi         - IMSI identifier
     * @param defaultEbi  - EBI
     * @param ueIpv4      - UE IPv4 Address
     * @param s1USgwTeid - SGW Tunnel Identifier
     * @param s1USgwIpv4 - SGW IPv4
     * @param sessionId   - Context Identifier
     * @param clientId    - Client Identifier
     * @param opId        - Operation Identifier
     */
    void createSession(
            byte topicId,
            BigInteger imsi,
            Short defaultEbi,
            Ip4Address ueIpv4,
            Long s1USgwTeid,
            Ip4Address s1USgwIpv4,
            Long sessionId,
            Long clientId,
            BigInteger opId
    );

    /**
     * Modifies Bearer.
     *
     * @param topicId        - DPN Topic ID
     * @param s1USgwIpv4    - SGW IPv4 Address
     * @param s1UEnodebTeid - ENodeB Tunnel Identifier
     * @param s1UEnodebIpv4 - ENodeB IPv4 Address
     * @param sessionId      - Context Identifier
     * @param clientId       - Client Identifier
     * @param opId           - Operation Identifier
     */
    void modifyBearer(
            byte topicId,
            Ip4Address s1USgwIpv4,
            Long s1UEnodebTeid,
            Ip4Address s1UEnodebIpv4,
            Long sessionId,
            Long clientId,
            BigInteger opId
    );

    /**
     * Deletes Mobility Session.
     *
     * @param topicId   - DPN Topic ID
     * @param sessionId - Context Identifier
     * @param clientId  - Client Identifier
     * @param opId      - Operation Identifier
     */
    void deleteSession(
            byte topicId,
            Long sessionId,
            Long clientId,
            BigInteger opId
    );

    /**
     * Creates the byte buffer to send ADC rules over ZMQ.
     *
     * @param topic        - DPN Topic ID
     * @param domainName  - domain
     * @param ip           - ipaddress/ipprefix (i.e. 127.0.0.1/32)
     * @param drop         - Drop if 1
     * @param ratingGroup - Rating Group
     * @param serviceId   - Service ID
     * @param sponsorId   - Sponsor ID
     */
    void sendAdcRules(
            Short topic,
            String domainName,
            String ip,
            Short drop,
            Long ratingGroup,
            Long serviceId,
            String sponsorId
    );
}
